package com.docus.server.collect.dept.view;

import com.docus.log.annotation.TrackRetryListener;
import com.docus.server.collect.ICollector;
import com.docus.server.collect.web.job.AbstractCollectJob;
import com.docus.server.entity.TaskConfig;
import com.docus.server.his.service.IHisService;
import com.docus.server.sys.common.pojo.dto.DeptDTO;
import com.docus.server.util.PeriodTime;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

/**
 * 主动view请求科室数据job
 *
 * @author linruifeng
 * @see AbstractCollectJob
 * @see com.docus.server.collect.IJob
 * @see ICollector
 */
@Component
public class DeptViewCollectJob extends AbstractCollectJob<DeptDTO> {
    @Resource
    private IHisService hisService;

    /**
     * xxl jobhandler 手工执行，全量执行
     */
    @XxlJob("startAllDeptViewCollect")
    @TrackRetryListener("startAllDeptViewCollect")
    public void startAllDeptViewCollect(String param) {
        super.startCollectAll(getJobParam(param));
    }

    /**
     * xxl jobhandler 配置定时启动，如果未完成任务跳过。增量收集
     */

    @XxlJob("startIncDeptViewCollect")
    @TrackRetryListener("startIncDeptViewCollect")
    public void startIncDeptViewCollect(String param) {
        super.startCollectIncrement(getJobParam(param));
    }

    @Override
    public void doHandle(List<DeptDTO> results, String param) {
        deptService.batchInsertOrUpdatePowerDept(results);
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<DeptDTO> execute(PeriodTime periodTime, TaskConfig taskConfig) {
        //每家医院按照固定的格式写sql，不同医院需要替换不同的sql即可。
        return hisService.getDeptListView(periodTime.getPeriodStartDate(), periodTime.getPeriodEndDate(),
            taskConfig.getPageNum(), taskConfig.getPageSize());
    }
}
